﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 * and the BEACON Center for the Study of Evolution in Action.
 * 
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;

namespace HeuristicLab.Algorithms.DataAnalysis {
  // returns the optimal value for the partition of rows stored in idx[startIdx] .. idx[endIdx] inclusive
  public delegate double LineSearchFunc(int[] idx, int startIdx, int endIdx);

  // represents an interface for loss functions used by gradient boosting
  // target represents the target vector  (original targets from the problem data, never changed)
  // pred   represents the current vector of predictions (a weighted combination of models learned so far, this vector is updated after each step)
  // weight represents a weight vector for rows (this is not supported yet -> all weights are 1)
  public interface ILossFunction {
    // returns the weighted loss of the current prediction vector 
    double GetLoss(IEnumerable<double> target, IEnumerable<double> pred, IEnumerable<double> weight);

    // returns an enumerable of the weighted loss gradient for each row
    IEnumerable<double> GetLossGradient(IEnumerable<double> target, IEnumerable<double> pred, IEnumerable<double> weight);

    // returns a function that returns the optimal prediction value for a subset of rows from target and pred (see LineSearchFunc delegate above)
    LineSearchFunc GetLineSearchFunc(IEnumerable<double> target, IEnumerable<double> pred, IEnumerable<double> weight);
  }
}


